import { csv, select } from 'd3';
import { scatterPlot } from './scatterPlot';
//Attaching csv URL
const csvUrl = [
  'https://gist.githubusercontent.com/',
  'GaddamThanmayeeReddy/', // User
  'bb60d6bca8435bfbf840f2bdd211b8e0/', // Id of the Gist
  'raw/19e580bb7176e962b3a3169188dc607d772c5925/', // commit
  'home.csv', // File name
].join('');
const parseRow = (d) => {
  d.table_length = +d.table_length;
  d.table_width = +d.table_width;
  d.chair_length = +d.chair_length;
  d.chair_width = +d.chair_width;
  return d;
};
//Width and Height are determined
//These values are constant and are set to
//the windows innerHeight and innerWidth
const width = window.innerWidth;
const height = window.innerHeight;
//appending the svg to body
const svg = select('body')
  .append('svg')
  .attr('width', width)
  .attr('height', height);

//Creating the async()
//The scatterplot is created
const main = async () => {
  const plot = scatterPlot()
    .width(width)
    .height(height)
    .data(await csv(csvUrl, parseRow))
    .xValue((d) => d.chair_width)
    .yValue((d) => d.table_length)
    .rxValue((d) => d.table_width)
    .ryValue((d) => d.chair_length)
    .furnitureValue((d) => d.furniture)
    .symbolValue((d) => d.furniture)
    .margin({
      top: 20,
      right: 20,
      bottom: 40,
      left: 60,
    })
    .radius(5);

  svg.call(plot);
  //Columns are initialized below
  const columns = [
    'chair_width',
    'table_width',
    'chair_length',
    'table_length',
  ];

  //Creating on-page text display
  //Showing colors based on furniture
  svg
    .append('rect')
    .attr('x', 120)
    .attr('y', 50)
    .attr('width', 150)
    .attr('height', 100)
    .attr('stroke', 'blue')
    .attr('fill', '#F3F3F3');
  svg
    .append('text')
    .attr('x', 300)
    .attr('y', 20)
    .text('Reusable D3 Scatter Plot')
    .style('font-size', '40px')
    .attr('alignment-baseline', 'middle')
    .style('fill', '#2F4F4F')
    .style('font-weight', 'bold');
  svg
    .append('text')
    .attr('x', 150)
    .attr('y', 70)
    .text('ikea')
    .style('font-size', '20px')
    .style('fill', '#C71585')
    .style('font-family', 'calibri')
    .attr('alignment-baseline', 'middle');
  svg
    .append('text')
    .attr('x', 150)
    .attr('y', 100)
    .text('oak')
    .style('font-size', '20px')
    .style('fill', '#00FA9A')
    .style('font-family', 'calibri')
    .attr('alignment-baseline', 'middle');
  svg
    .append('text')
    .attr('x', 150)
    .attr('y', 130)
    .text('durian')
    .style('font-size', '20px')
    .style('fill', '#A52A2A')
    .style('font-family', 'calibri')
    .attr('alignment-baseline', 'middle');

  //Changing the scatterplot after a specific time
  let i = 0;
  setInterval(() => {
    plot.xValue((d) => d[columns[i % columns.length]]);
    svg.call(plot);
    i++;
  }, 1000);
};
main();
