//Requirements are imported from d3
import {
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
  symbols,
  symbol,
  scaleOrdinal,
} from 'd3';

//initializing the varibles using "let"
export const scatterPlot = () => {
  let width;
  let height;
  let data;
  let xValue;
  let yValue;
  let symbolValue;
  let rxValue;
  let ryValue;
  let furnitureValue;
  let margin;
  let radius;

  const my = (selection) => {
    //Create linear scale for x (x-axis)
    const x = scaleLinear()
      .domain(extent(data, xValue))
      .range([margin.left, width - margin.right]);
    //Create linear scale for y (y-axis)
    const y = scaleLinear()
      .domain(extent(data, yValue))
      .range([height - margin.bottom, margin.top]);

    const symbolScale = scaleOrdinal()
      .domain(data.map(symbolValue))
      .range(symbols);
    //Assigning colors to furniture
    const marker = (d) => {
      switch (d) {
        case 'ikea':
          return '#C71585';
        case 'oak':
          return '#00FA9A';
        default:
          return '#A52A2A';
      }
    };
    //Symbol Generator is created
    const symbolGenerator = symbol();
    //Creating marks - to be plotted
    const marks = data.map((d) => ({
      x: x(xValue(d)),
      y: y(yValue(d)),
      furniture: furnitureValue(d),
      color: marker(furnitureValue(d)),
      rx: rxValue(d),
      ry: ryValue(d),
      pathD: symbolGenerator.type(
        symbolScale(symbolValue(d)),
      )(),
    }));

    //Attributes are created
    const circles = selection
      .selectAll('path')
      .data(marks)
      .join('path')
      .attr('cx', (d) => d.x)
      .attr('cy', (d) => d.y)
      .attr('fill', (d) => d.color)
      .attr('rx', (d) => d.rx)
      .attr('ry', (d) => d.ry)
      .attr('r', radius)
      .attr('d', (d) => d.pathD)
      .attr('transform', (d) => `translate(${d.x},${d.y})`);
    //Y-axis Selection
    selection
      .selectAll('.y-axis')
      .data([null])
      .join('g')
      .attr('class', 'y-axis')
      .attr('transform', `translate(${margin.left},0)`)
      .call(axisLeft(y));
    //X-axis Selection
    selection
      .selectAll('.x-axis')
      .data([null])
      .join('g')
      .attr('class', 'x-axis')
      .attr(
        'transform',
        `translate(0,${height - margin.bottom})`,
      )
      .call(axisBottom(x));
  };

  my.width = function (_) {
    return arguments.length ? ((width = +_), my) : width;
  };

  my.height = function (_) {
    return arguments.length ? ((height = +_), my) : height;
  };

  my.data = function (_) {
    return arguments.length ? ((data = _), my) : data;
  };

  my.xValue = function (_) {
    return arguments.length ? ((xValue = _), my) : xValue;
  };

  my.yValue = function (_) {
    return arguments.length ? ((yValue = _), my) : yValue;
  };
  my.rxValue = function (_) {
    return arguments.length ? ((rxValue = _), my) : rxValue;
  };

  my.ryValue = function (_) {
    return arguments.length ? ((ryValue = _), my) : ryValue;
  };

  my.furnitureValue = function (_) {
    return arguments.length
      ? ((furnitureValue = _), my)
      : furnitureValue;
  };

  my.symbolValue = function (_) {
    return arguments.length
      ? ((symbolValue = _), my)
      : symbolValue;
  };

  my.margin = function (_) {
    return arguments.length ? ((margin = _), my) : margin;
  };

  my.radius = function (_) {
    return arguments.length ? ((radius = +_), my) : radius;
  };

  return my;
};
